#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

#################################################################################
# Created       : Martin Foster
# Modified      : 06-Aug-2006
#################################################################################
#
# Home - Home page component for Ethereal Realms 
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3-3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

use CGI;							# Common gateway interface
use CGI::Carp;							# CGI Error logs
use File::Copy;							# File copy handle
use Number::Format qw(:subs);					# Number formatting library
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Home;						# Homepage/file component
use Ethereal::Image;						# Image handling library
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control
use Ethereal::Template;						# Template handling

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 8192;					# Maximum posts
$CGI::DISABLE_UPLOADS = 0;					# Disable uploads

#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $hp;								# Homepage component
my $login;							# Login handle
my $param;							# Parameter handle
my $tmpl;							# Template handle

my $expire;							# Expired homepages
my $url;							# Self referencing link

my @list;							# File selection
my @pass;							# Passed params
my @upload;							# Uploaded files

my %args;							# Embeded parameters
my %contrib;							# Contributor page
my %links;							# Hash of links


#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new Ethereal::Database();
	
	# Kill switch
	exit if $database->{'COMP_HOME'} eq 'false';

	# Connect and fetch
	$database->Connect($cgi);


	# Retreive hash
	$database->GetHashContrib(\%contrib);


	# Advanced retreivals
	# Handle uploads
	for (my $cnt=1; $cnt <= $contrib{'SetUpload'}; $cnt++)
	{
		# Add to array
		$upload[$cnt] = $cgi->upload("FUPLOAD$cnt");
	}

	# Removals
	@list = $cgi->param('FSELECT');	


	# Homepage and Parameter instances
	$hp    = new Ethereal::Home($database, $contrib{'SetHomeRoot'});
	$param = new Ethereal::Param($database, $cgi);
	$param->GetParam();

	# Create instance
	$tmpl = new Ethereal::Template(\%contrib);

	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param, \%contrib);

	# Enable custom header and footer
	$login->{'HEADER'} = $database->DocumentGetContribHeader();
	$login->{'FOOTER'} = $database->DocumentGetContribFooter();


	# Verify login and continue with system
	if ($login->GetVerificationNormal())
	{
		# Document header
		print $login->{'HEADER'}, "\n";
			
		# Expired or not?
		($expire) = $database->DataGetPuppeteerExpire($param->{'USER'});

		# Account still valid
		if ($expire eq 'no')
		{
			# Fetch self referencing link
			$url = $cgi->url(-absolute=>1);

			# Base parameters
			%args = (
				USER  => $param->{'USER'},
				CRYPT => $param->{'CRYPT'}
			 );

			# Appropriate parameters
			$links{'LMAIN'} = $url . $param->EmbedInline(%args);
			$links{'LLIST'} = $url . $param->EmbedInline(%args, LISTING=>'True');
			$links{'LSIZE'} = $url . $param->EmbedInline(%args, RESIZER=>'True');
			$links{'LUPLD'} = $url . $param->EmbedInline(%args, UPLOADS=>'True');

			# Quick param
			@pass = ($database, $cgi, $param, $tmpl, $hp, \%contrib, \%links);


			# Make sure Directory is sane 
			if (DirHandler(@pass))
			{
				# Directory listing
				if (defined($param->{'LISTING'}))
				{
					# Display information
					DirListing(@pass);
				}

				# Image resizing
				elsif (defined($param->{'RESIZER'}))
				{
					# Handling of folder
					FileResize(@pass, \@list);
				}


				# File uploads
				elsif (defined($param->{'UPLOADS'}))
				{
					# Handling of uploads
					FileUpload(@pass);
				}

				else
				{
					# Handling of folder
					if (FileEdit(@pass, \@list))
					{
						# Default display
						HomeHandler(@pass, \@upload, \@list); 
					}
				}
			}
		}
		else
		{
			# Display disabled message
			$tmpl->Show('TmplHomeExpire');
		}

		# Display footer
		print $login->{'FOOTER'}, "\n";
	}


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Directory Handler 
#
# This will make sure that certain components of Ethereals Home page system,
# primarily the home page directory is in order.   This means creation, prompt
# or whatever is needed to get it going. 

sub DirHandler 
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template handle
	my $hp       = shift;					# Homepage handle
	my $contrib  = shift;					# Home page hash handle 
	my $links    = shift;					# Hash of links
	my $message  = shift;					# Error message

	my $dir;						# Directory

	my $header;						# Header file
	my $readme;						# Readme file


	#####################
	# Program area

	# Retreive DIR
	($dir) = $database->DataGetPuppeteerHome($param->{'USER'});


 	# Determin status 
	# Directory defined 
	if (defined($dir))
	{
		# Set Basedir
		$hp->{'BASEDIR'} = $contrib->{'SetHomeRoot'} . $hp->{'SEP'} . $dir;

		# Ensure existence
		unless (-d $hp->{'BASEDIR'})
		{
			# Create and verify
			unless ($hp->DirCreate($hp->{'BASEDIR'}))
			{
				# Display error message
				$tmpl->Show('TmplHomeError',
					MMESSAGE => $contrib->{'MsgFail'},
					MERROR   => $!
				 );

				# Reset Database
				$database->Quick("UPDATE Puppeteer 
					SET   PuppeteerHome=NULL 
					WHERE PuppeteerLogin=?", 

				 $param->{'USER'});	
				
				# Return False
				return 0;
			} 
		}


		# Retreive header and readme
		($header) = $database->DocumentGetHomeHeader();
		($readme) = $database->DocumentGetHomeReadme();

		# Write contents
		$hp->FileWrite($contrib->{'SetHEADER'}, $header);
		$hp->FileWrite($contrib->{'SetREADME'}, $readme);

		# Return true
		return 1;
	}

	# Directory creation
	elsif ((defined($param->{'DCREATE'}))
	    && ($param->{'DCREATE'} !~ /\W+/g))
	{
		# Parameter handling
		$dir = $param->{'DCREATE'};
		delete $param->{'DCREATE'};

		# Set Basedir
		$hp->{'BASEDIR'} = $contrib->{'SetHomeRoot'} . $hp->{'SEP'} . $dir;


		# Ensure existence
		unless (-d $hp->{'BASEDIR'})
		{
			# Create and verify
			# Successful
			if ($hp->DirCreate($hp->{'BASEDIR'}))
			{
				# Retreive header and readme
				($header) = $database->DocumentGetHomeHeader();
				($readme) = $database->DocumentGetHomeReadme();

				# Write contents
				$hp->FileWrite("$hp->{BASEDIR}$hp->{SEP}$contrib->{SetHEADER}", $header);
				$hp->FileWrite("$hp->{BASEDIR}$hp->{SEP}$contrib->{SetREADME}", $readme);

				# Update Database
				$database->Quick("UPDATE Puppeteer 
					SET   PuppeteerHome=? 
					WHERE PuppeteerLogin=?", 

				 $dir,
				 $param->{'USER'});
				
				# Return True
				return 1;
			} 

			# Fail
			else
			{
				# Display error message
				$tmpl->Show('TmplHomeError',
					MMESSAGE => $contrib->{'MsgFail'},
					MERROR   => $!
				 );
			}
		}
		else
		{ DirHandler($database, $cgi, $param, $tmpl, $hp, $contrib, $contrib->{'MsgExists'}); }
		
		# Return false
		return 0; 
	} 

	# New page setup
 	else	
	{
		# Safety
		$message = (defined($message)) ? $message : '';


		# Start form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat()), "\n";


		# Display template
		$tmpl->Show('TmplHomeCreate',
			MMESSAGE => $message,
			WCREATE  => $cgi->textfield('DCREATE', '', 55, 55),
			WSUBMIT  => $cgi->submit($contrib->{'TxtCreate'})
		 );

		# Embed and close form
		print $cgi->end_form(), "\n";

		# Return false
		return 0;
	}
}

#####################
# Directory Listing
#
# Used to show a user all files in the folder and allow them to select it
# if needed.  This is used if directory indexing has been disabled.

sub DirListing
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template handle
	my $hp       = shift;					# Homepage handle
	my $contrib  = shift;					# Home page hash handle 
	my $links    = shift;					# Hash of links

	my $bytes;						# Individual size
	my $count;						# File count
	my $dir;						# Users Directory
	my $file;						# Individual file
	my $size;						# Size of files

	my $list;						# List template

	my @files;						# File listing
	my @listing;						# Listing of files


	#####################
	# Program Area

	# Cleanup
	$param->Cleanup('LISTING');


	# Data retreival
	# Retreive DIR
	($dir) = $database->DataGetPuppeteerHome($param->{'USER'});

	# Base directory
	$hp->{'BASEDIR'} = $contrib->{'SetHomeRoot'};


	# Retreive file listing
	($size, $count) = $hp->FileListing($dir, \@files, $contrib); 

	# Initialization
	$list = '';

	# Cycle and list files
	foreach $file (sort(@files))
	{
		# Retreive file size
		$bytes = -s $contrib->{'SetHomeRoot'} . '/' . $dir . '/' . $file;

		# Filename is escaped
		$file = $cgi->escape($file);

		# Display Filename
		$list .= $tmpl->Pass('TmplHomeListingItem',
			MLINK  => $contrib->{'SetHomeUrl'} . '/' . $dir . '/' . $file,
			MNAME  => $file,
			MBYTES => format_bytes($bytes)
		 );
	}

	# Display main template
	$tmpl->Show('TmplHomeListing',
		LMAIN    => $links->{'LMAIN'},
		LLIST    => $links->{'LLIST'},
		LSIZE    => $links->{'LSIZE'},
		LUPLD    => $links->{'LUPLD'},
		LSTITEMS => $list,
		MCOUNT   => $count
	 );
}


#####################
# File Conflict
#
# File conflict is there to prevent any chance of files being overriden
# by file uploads and resize handling.

sub FileConflict
{
	#####################
	# Data members
	my $dir  = shift;					# Directory name
	my $file = shift; 					# Filename

	my $fn;							# Filename
	my $num = 0;						# Number


	#####################
	# Program area

	# Acceptable file name
	if (-f "$dir/$file")
	{
		do
		{
			# Increment
			$num++;

			# Assign
			$fn = $file; 

			# Rename
			$fn =~ s/^(.+)\.(.*)$/$1-$num.$2/s;
		} 
	 	while (-f "$dir/$fn");

		# Re-assign
		$file = $fn;
	}

	# Return
	return $file;
}


#####################
# File Edit 
#
# File edit, will be called before the default script and will be used in cases
# where the option to edit a file has been clicked.   If it is not needed
# it will return true and the default will kick in. 

sub FileEdit 
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template handler
	my $hp       = shift;					# Homepage handle
	my $contrib  = shift;					# Home page hash handle 
	my $links    = shift;					# Hash of links
	my $list     = shift;					# Selected files

	my $contents;						# Contents of file
	my $dir;						# Directory name
	my $file;						# File name
	my $message;						# Status message


	#####################
	# Program area

	# Initial check
	return 1 unless (defined($param->{$contrib->{'TxtEdit'}}));


	# Retreive directory handle 
	($dir)  = $database->DataGetPuppeteerHome($param->{'USER'});

	# Filename
	($file) = $param->{'FSELECT'};


	# Set base directory
	$hp->{'BASEDIR'} = $contrib->{'SetHomeRoot'} . $hp->{'SEP'} . $dir;


	# Operations
	# Resize
	if (defined($param->{'FEDIT'}))
	{
		# Correct size
	 	if (length($param->{'FEDIT'}) > $contrib->{'SetEditMin'})
		{
			# Write file	
			$hp->FileWrite($file, $param->{'FEDIT'});

			# Message display
			$message = $contrib->{'MsgEdit'};	
		}

		else
		{
			# Warn of min file size
			$message = $tmpl->Pass('MsgChar', 'MCHAR', $contrib->{'SetEditMin'});
		}

		# Cleanup
		$param->Cleanup('FEDIT');
	}


	# File load and display
	if ((defined($file))
	 && ($file =~ /\.(css|htm|html|shtml|txt)$/i)
	 && (open(EDIT, $hp->{'BASEDIR'} . $hp->{'SEP'} . $file)))
	{
		# Read from file
		while (<EDIT>) { $contents = "$contents$_"; }

		# Close
		close(EDIT);


		# Safety
		$message = (defined($message)) ? $message : '';

		# Generate wigets
		# Edit box
		my $weditor = $cgi->textarea(
			-name=>'FEDIT',
			-rows=>10,
			-columns=>75,
			-wrap=>'OFF',
			-default=>$contents);

		# Submit entry
		my $wsubmit = $cgi->submit($contrib->{'TxtEdit'});


		# HTML Display
		# Start form
		print $cgi->start_form();
		print $param->EmbedNormal(
			USER=>$param->{'USER'},
			CRYPT=>$param->{'CRYPT'},
			FSELECT=>$file
		 );


		# Display Template
		$tmpl->Show('TmplHomeEdit',
			LMAIN    => $links->{'LMAIN'},
			LLIST    => $links->{'LLIST'},
			LSIZE    => $links->{'LSIZE'},
			LUPLD    => $links->{'LUPLD'},
			MMESSAGE => $message,
			WEDITOR  => $weditor,
			WSUBMIT  => $wsubmit
		 );


		# Embed and close form
		print $cgi->end_form(), "\n";


		# Retrun false
		return 0;
	}

	# Retrun true
	return 1;
}


#####################
# File Resize 
#
# File resize will allow a user to select a file and resize it to an appropriate
# demension.   Due to the nature of this, some steps will be taken to only allow
# images, though nothing is certain.

sub FileResize 
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template handler
	my $hp       = shift;					# Homepage handle
	my $contrib  = shift;					# Home page hash handle 
	my $links    = shift;					# Hash of links
	my $list     = shift;					# Selected files

	my $dir;						# Directory name
	my $file;						# File name
	my $image;						# Image handle
	my $message;						# Status message

	my @files;						# Files list
	my @sorted;						# Sorted files


	#####################
	# Program area

	# Retreive directory handle 
	($dir) = $database->DataGetPuppeteerHome($param->{'USER'});

	# Set basedir
	$hp->{'BASEDIR'} = $contrib->{'SetHomeRoot'} . $hp->{'SEP'} . $dir;


	# Operations
	# Resize
	if (defined($param->{$contrib->{'TxtResize'}}))
	{
		# First file name
		$file = shift(@{$list});

		# Safety checks
		if (($file ne $contrib->{'TxtNone'})
		 && ($param->{'FMAX'} =~ /^\d+$/)
		 && ($param->{'FMAX'} > 0)
		 && ($param->{'FMAX'} <= $contrib->{'SetResize'}))
		{
			# Create instance
			$image = new Ethereal::Image($database, $hp->{'BASEDIR'});


			# Create copy
			# Filenames
			my $old = $file;
			my $new = FileConflict($hp->{'BASEDIR'}, $file);

			# Copy to resize
			copy("$hp->{BASEDIR}$hp->{SEP}$old", "$hp->{BASEDIR}$hp->{SEP}$new");


			# Resize
			$image->Set($new, $param->{'FMAX'});

			# Set Message
			$message = $contrib->{'MsgResize'};
		}


		# Cleanup
		$param->Cleanup('FMAX','FSELECT');
	}



	# Reset basedir
	$hp->{'BASEDIR'} = $contrib->{'SetHomeRoot'};


	# Data handling
	# Retreive file listing
	$hp->FileListing($dir, \@files, $contrib); 

	# Exclude
	foreach $file (@files)
	{
		# Push onto array
		# Specific types only
		push(@sorted, $file) if $file =~ /\.(gif|jpe|jpeg|jpg|pcx|psd|png|tiff|tga)$/i;
	}


	# Sort and prepare 
	@sorted = sort(@sorted);
	unshift(@sorted, $contrib->{'TxtNone'});


	# Generate widget
	# Sorted list
	my $sorted  = $cgi->scrolling_list('FSELECT', \@sorted, $sorted[0], 15);

	# Max dimension
	my $wmax    = $cgi->textfield('FMAX', '300', 5, 3); 

	# Submission button
	my $wsubmit = $cgi->submit($contrib->{'TxtResize'});



	# Safety
	$message = (defined($message)) ? $message : '';


	# HTML Display
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";


	# Display template
	$tmpl->Show('TmplHomeResize',
		LMAIN    => $links->{'LMAIN'},
		LLIST    => $links->{'LLIST'},
		LSIZE    => $links->{'LSIZE'},
		LUPLD    => $links->{'LUPLD'},
		MMESSAGE => $message,
		WSORTED  => $sorted,
		WMAX     => $wmax,
		WSUBMIT  => $wsubmit
	 ); 


	# Embed and close form
	print $cgi->end_form(), "\n";
}


#####################
# File Upload
#
# File upload window, which will allow the main window to handle the
# entire process.  As a result, this window is literally a one time
# only operation.

sub FileUpload
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template handle
	my $hp       = shift;					# Homepage handle
	my $contrib  = shift;					# Home page hash handle 
	my $links    = shift;					# Hash of links

	my $cnt;						# File count
	my $dir;						# Users Directory
	my $item;						# Items list
	my $size;						# Space used

	my @files;						# File listing


	#####################
	# Program Area

	# Cleanup
	$param->Cleanup('UPLOADS');


	# Data retreival
	# Retreive DIR
	($dir) = $database->DataGetPuppeteerHome($param->{'USER'});

	# Base directory
	$hp->{'BASEDIR'} = $contrib->{'SetHomeRoot'};


	# Retreive file listing
	($size) = $hp->FileListing($dir, \@files, $contrib); 


	# Initialize
	$item = '';

	# Addition of files slots
	if ($size < ($contrib->{'SetSize'} * 1048576))
	{
		# Cycle through
		for ($cnt=1; $cnt <= $contrib->{'SetUpload'}; $cnt++)
		{
			# Display addition area
			$item .= $tmpl->Pass('TmplHomeUploadItem',
				MCOUNT  => $cnt,
				WUPLOAD => $cgi->filefield("FUPLOAD$cnt", '', 40, 140)
			 );
		}
	}


	# Display information
	# Start form
	print $cgi->start_multipart_form();
	print $param->EmbedNormal($param->Flat()), "\n";

	# Display template
	$tmpl->Show('TmplHomeUpload',
		LMAIN    => $links->{'LMAIN'},
		LLIST    => $links->{'LLIST'},
		LSIZE    => $links->{'LSIZE'},
		LUPLD    => $links->{'LUPLD'},
		LSTITEMS => $item,
		WUPLOAD  => $cgi->submit($contrib->{'TxtUpload'})
	 );

	# End form
	print $cgi->end_form(), "\n";
}


#####################
# HomeHandler 
#
# Provides the interface to see all files within their home folders as well
# as allow for additions and removals as one sees fit.  

sub HomeHandler
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template handler
	my $hp       = shift;					# Homepage handle
	my $contrib  = shift;					# Home page hash handle 
	my $links    = shift;					# Hash of links
	my $upload   = shift;					# File upload hash
	my $list     = shift;					# Selected files

	my $count;						# File listing
	my $dir;						# Directory
	my $size;						# File size

	my $contents;						# File contents
	my $file;						# File name
	my $handle;						# File handle
	my $type;						# File type

	my $err;						# Error message
	my $message;						# Status message

	my @files;						# Files list
	my @sorted;						# Sorted files


	#####################
	# Program area

	# Retreive directory handle 
	($dir) = $database->DataGetPuppeteerHome($param->{'USER'});

	# Set basedir
	$hp->{'BASEDIR'} = $contrib->{'SetHomeRoot'} . $hp->{'SEP'} . $dir;


	# Operations
	# Upload field handle
	for (my $cnt=1; $cnt <= $contrib->{'SetUpload'}; $cnt++)
	{
		# Create handle
		$handle = $upload->[$cnt];

		# Worth a look?
		if (defined($handle))
		{	
			# Retreive MIME Type
			$type = $cgi->uploadInfo($param->{"FUPLOAD$cnt"})->{'Content-Type'};

			# Filename handling
			$file = $param->{"FUPLOAD$cnt"}; 
			$file =~ m!([^/:\\]*)$!;
			$file = $1;  

			# Only allow specific MIME types
			if (($type =~ /^image/i) || ($type =~ /^text/i))
			{
				unless (($file =~ /^\./)
				  || ($file eq $contrib->{'SetHEADER'})
				  || ($file eq $contrib->{'SetREADME'}))
				{
					# Acceptable file name
					$file = FileConflict($contrib->{'SetHomeRoot'} . '/' . $dir, $file);


					# Retreive information
					$contents = '';
					while (<$handle>) { $contents = "$contents$_"; }
			
					# Write information
					$err = $hp->FileWrite($file, $contents); 

					# Happy land
					$message = $contrib->{'MsgCreate'};
				}
				# Otherwise error message
				else
				{ $message = $contrib->{'MsgFail'}; }
			}
			# Otherwise error message
			else
			{ $message = $contrib->{'MsgFail'}; }
		}

		# Cleanup
		$param->Cleanup("FUPLOAD$cnt");
	}

	# Remove files
	if ((defined($param->{$contrib->{'TxtDelete'}}))
	 && (defined($param->{'CONFIRM'})))
	{
		# Cycle
		foreach $file (@{$list})
		{
	 		if ($file ne $contrib->{'TxtNone'})
			{
				# Remove file
				$hp->FileUnlink($file);

				# Assign message
				$message = $contrib->{'MsgDelete'};
			}
		}

		# Cleanup
		$param->Cleanup('FSELECT');	
	}

	# Always clean
	$param->Cleanup('CONFIRM');


	# Rename
	if (defined($param->{$contrib->{'TxtRename'}}))
	{
		# First file name
		my $new = $param->{'FRENAME'};
		my $old = shift(@{$list});

		# Safety checks
		if (($old ne $contrib->{'TxtNone'})
		 && ($new =~ /^\w[\w\.\-]+$/)
		 && (!-f $contrib{'SetHomeRoot'} . $hp->{'SEP'} . $dir . $hp->{'SEP'} . $new))
		{
			# Rename file
			rename($contrib{'SetHomeRoot'} . $hp->{'SEP'} . $dir . $hp->{'SEP'} . $old,
			       $contrib{'SetHomeRoot'} . $hp->{'SEP'} . $dir . $hp->{'SEP'} . $new
		         );

			# Assign message
			$message = $contrib->{'MsgRename'};
		}

		# Cleanup
		$param->Cleanup('FSELECT', $contrib->{'TxtRename'});
	}



	# Reset basedir
	$hp->{'BASEDIR'} = $contrib->{'SetHomeRoot'};


	# Data handling
	# Retreive file listing
	($size, $count) = $hp->FileListing($dir, \@files, $contrib); 

	# Formatting
	$size  = format_bytes($size, 2);
	$count = format_number($count, 0, 0);

	# Sort and prepare 
	@sorted = sort(@files);
	unshift(@sorted, $contrib->{'TxtNone'});


	# Safety
	$message = (defined($message)) ? $message : '';


	# Generate widgets
	# Sorted file list
	my $sorted = $cgi->scrolling_list('FSELECT', \@sorted, $sorted[0], 15, 'true');

	# File list control wigets
	my $buttons = $cgi->submit($contrib->{'TxtEdit'}) . ' <BR><BR> '
	  . $cgi->submit($contrib->{'TxtDelete'}) . ' &nbsp; ' 
	  . $cgi->checkbox(-name=>'CONFIRM', -label=>'');

	# Rename options
	my $wnewname = $cgi->textfield('FRENAME', '', 35, 35);
	my $wrename  = $cgi->submit($contrib->{'TxtRename'});


	# HTML Display
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";


	# Display template
	$tmpl->Show('TmplHome',
		LMAIN    => $links->{'LMAIN'},
		LLIST    => $links->{'LLIST'},
		LSIZE    => $links->{'LSIZE'},
		LUPLD    => $links->{'LUPLD'},
		MMESSAGE => $message,
		MTSIZE   => $size,
		MTCOUNT  => $count,
		MTMAX    => $contrib->{'SetSize'},
		WSORTED  => $sorted,
		WBUTTONS => $buttons,
		WNEWNAME => $wnewname,
		WRENAME  => $wrename
	 );

	# Embed and close form
	print $cgi->end_form(), "\n";
}
