#################################################################################
# Created       : Martin Foster
# Modified      : 05-May-2003
#################################################################################
#
# Table - Modules part of Ethereal Realms, designed to simplify table creation  
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

package Ethereal::Table;					# Package name

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	my $class = shift;					# Self reference
	my $cgi   = shift;					# CGI handle

	my $self = {CGI => $cgi};
	bless $self, $class;					# Bless $self into designated class
	return $self;
}



#####################
# Make Table
#
# Table contruction sub-routine, designed to hopefully cut down on code for certain
# apects of scripts needed it.

sub Make
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $choice   = shift;					# Decision to make

	my $cgi      = $self->{CGI};				# CGI handle
	my $width;						# Table width
	my $align;						# Table alignment

	#####################
	# Program area


	# Top of table
	if ($choice == 0)
	{
		# Verify value
		$width = shift;
		$width = (defined($width)) ? $width : 500;

		# Table top
		print "<CENTER>\n<TABLE WIDTH=\"$width\" COLS=3>\n"; 
	}

	# blank row of table
	elsif ($choice == 1)
	{
	  print " <TR>\n",
	        "  <TD HEIGHT=15 COLPSAN=3>&nbsp;</TD>\n",
	        " </TR>\n"; 
	}


	# Filled row of table (Invalid)
	elsif ($choice == 2) 
	{
		# Define alignmnet
		$align = $_[2];
		$align = (defined($align)) ? $align : 'top';

		# Top of row
		print " <TR VALIGN=\"$align\">\n";

		# Body
		print "  <TD>$_[0]</TD>\n";
		print "  <TD WIDTH=10><FONT COLOR=\"\#FF0000\"><B>*</B></FONT></TD>\n";
		print "  <TD>$_[1]</TD>\n";

		# Bottom of row
		print " </TR>\n";
	}

	# Filled row of table (Done)
	elsif ($choice == 3)
	{
		# Define alignmnet
		$align = $_[2];
		$align = (defined($align)) ? $align : 'top';

		# Top of row
		print " <TR VALIGN=\"$align\">\n";

		# Body
		print "  <TD>$_[0]</TD>\n";
		print "  <TD WIDTH=10>&nbsp;</TD>\n";
		print "  <TD>$_[1]</TD>\n";

		# Bottom of row
		print " </TR>\n";
	}

	# Filled row of table (Changed)
	elsif ($choice == 4)
	{
		# Define alignmnet
		$align = $_[2];
		$align = (defined($align)) ? $align : 'top';

		# Top of row
		print " <TR VALIGN=\"$align\">\n";

		# Body
		print "  <TD>$_[0]</TD>\n";
		print "  <TD WIDTH=10><FONT COLOR=\"\#009900\"><B>*</B></FONT></TD>\n";
		print "  <TD>$_[1]</TD>\n";

		# Bottom of row
		print " </TR>\n";
	}


	# Tertiary Rows
	elsif ($choice == 5)
	{
		# Define alignmnet
		$align = $_[3];
		$align = (defined($align)) ? $align : 'top';

		# Top of row
		print " <TR VALIGN=\"$align\">\n";

		# Body
		print "  <TD>$_[0]</TD>\n";
		print "  <TD>$_[1]</TD>\n";
		print "  <TD>$_[2]</TD>\n";

		# Bottom of row
		print " </TR>\n";
	}

	# Single row
	elsif ($choice == 6)
	{
		# Define alignmnet
		$align = $_[1];
		$align = (defined($align)) ? $align : 'top';

		# Top of row
		print " <TR VALIGN=\"$align\">\n";

		# Body
		print "  <TD COLSPAN=3>$_[0]</TD>\n";

		# Bottom of row
		print " </TR>\n";
	}

        # bottom of table
        elsif ($choice == 7)
        { print "</TABLE>\n</CENTER>\n"; }
}

# Alias for top
sub MakeTop
{ Make(shift, 0, shift); }

# Alias for bottom
sub MakeBottom
{ Make(shift, 7); }

# Alias for Blank rows
sub MakeBlank
{ Make(shift, 1); }

# Alias for Invalid entries
sub MakeInvalid
{ Make(shift, 2, @_); }

# Alias for Valid entries
sub MakeValid
{ Make(shift, 3, @_); }

# Alias for Changed entries
sub MakeChanged
{ Make(shift, 4, @_); }

# Alis for all three columns
sub MakeTertiary
{ Make(shift, 5, @_); }

# Alias for single
sub MakeSingle
{ Make(shift, 6, @_); }
						
1;								# Return true
