#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

################################################################################
# Created       : Martin Foster
# Modified      : 28-Aug-2004
################################################################################
#
# Comm update - Changes the comm panel from one method of handling to another
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler

################################################################################
# Data Members
################################################################################
my $site;							# Site schema	

my $database;							# Database handle
my $statement;							# Defined query

my $panel;							# Default template
my $realm;							# Realm name

my @sites;							# List of sites

my %realm;							# RealmHash

################################################################################
# Program Area
################################################################################

	# Handling of defaults
	$site = ((defined($ARGV[0])) && ($ARGV[0] =~ /^-\w+/)) ? shift(@ARGV) : 'all';

	# Connect to Database
	$database = new Ethereal::Database();
	$database->Connect();


	# Population of array
	# Global listing
	if ($site eq 'all')
	{
		# Pull list of active sites
		$database->GetList(\@sites, "SELECT SiteSchema
			FROM public.Search
			GROUP BY SiteSchema");
	}

	# Unique site only
	else
	{
		# Populate hash
		push(@sites, $site);
	}

	# Loop as necessary
	foreach $site (@sites)
	{
		# Display
		print "Working on site: $site\n";

		# Set search paths
		$database->Write("SET SEARCH_PATH TO $site,public");


		# Hash
		# Display
		print "  Setting hash\n\n";

		# Pull
		$database->GetHashRealm(\%realm);
		

		# Display
		print "  Default template..";

		# Pull default
		($panel) = $database->DataGet("SELECT TagData
			FROM $site.Tag
	       		WHERE TagName=?
	 		  AND TagType=?", 'RealmPanel', 'template');

		# Replace links
		# Consisent
		$panel =~ s/LPORTAL/HREF=\"$realm{'LnkPortal'}BASEPARAM\" TARGET=\"_BLANK\" ACCESSKEY=\"h\"/gs;

		# Standard links
 		$panel =~ s/LBOARDS/HREF=\"$realm{LnkBoard}GENPARAM\" TARGET=\"REALMS\" TABINDEX=\"8\" GENJSCODE/gs;
 		$panel =~ s/LIGNORE/HREF=\"$realm{LnkIgnore}GENPARAM\" TARGET=\"REALMS\" TABINDEX=\"9\" GENJSCODE/gs;
 		$panel =~ s/LIMAILS/HREF=\"$realm{LnkMail}BASEPARAM\" TARGET=\"REALMS\" TABINDEX=\"10\" GENJSCODE/gs;

		# Disjoined
 		$panel =~ s/LCONFIG/HREF=\"$realm{LnkPuppet}GENPARAM\" TARGET=\"CONFIG\" TABINDEX=\"4\" GENJSCODE/gs;
 		$panel =~ s/LPRIVTE/HREF=\"$realm{LnkPrivate}GENPARAM\" TARGET=\"PRIVTE\" TABINDEX=\"5\" GENJSCODE/gs;
 		$panel =~ s/LDIEROL/HREF=\"$realm{LnkRoll}GENPARAM\" TARGET=\"DIEROL\" TABINDEX=\"6\" GENJSCODE/gs;
 		$panel =~ s/LWHOSON/HREF=\"$realm{LnkWho}GENPARAM\" TARGET=\"WHOSON\" TABINDEX=\"7\" GENJSCODE/gs;

		# Helper links
 		$panel =~ s/LCOLORS/HREF=\"$realm{LnkColors}GENPARAM\" TARGET=\"REALMS\" ACCESSKEY=\"g\" GENJSCODE/gs;
		$panel =~ s/LZOOMER/HREF=\"$realm{LnkZoom}GENPARAM\" TARGET=\"ZOOMER\" ACCESSKEY=\"z\" GENJSCODE/gs;


		# Updating
		print " Updating..";

		# Changes to handler
		$database->Quick("UPDATE $site.Tag
			SET TagData=?
			WHERE TagName=?
			  AND TagType=?", $panel, 'RealmPanel', 'template');

		print " Done!\n\n";


		# Loop and get all of the realms specific
		$database->Pull(\$statement, "Select RealmName, DesignContent
			FROM $site.RealmDesign
			WHERE TagName=?", 'RealmPanel');

		# Loop
		while (($realm, $panel) = $statement->fetchrow())
		{
			# Display
			print "  Realm: $realm..";
	
			# Replace links
			# Consisent
			$panel =~ s/LPORTAL/HREF=\"$realm{'LnkPortal'}BASEPARAM\" TARGET=\"_BLANK\" ACCESSKEY=\"h\"/gs;

			# Standard links
 			$panel =~ s/LBOARDS/HREF=\"$realm{LnkBoard}GENPARAM\" TARGET=\"REALMS\" TABINDEX=\"8\" GENJSCODE/gs;
 			$panel =~ s/LIGNORE/HREF=\"$realm{LnkIgnore}GENPARAM\" TARGET=\"REALMS\" TABINDEX=\"9\" GENJSCODE/gs;
 			$panel =~ s/LIMAILS/HREF=\"$realm{LnkMail}BASEPARAM\" TARGET=\"REALMS\" TABINDEX=\"10\" GENJSCODE/gs;

			# Disjoined
 			$panel =~ s/LCONFIG/HREF=\"$realm{LnkPuppet}GENPARAM\" TARGET=\"CONFIG\" TABINDEX=\"4\" GENJSCODE/gs;
 			$panel =~ s/LPRIVTE/HREF=\"$realm{LnkPrivate}GENPARAM\" TARGET=\"PRIVTE\" TABINDEX=\"5\" GENJSCODE/gs;
 			$panel =~ s/LDIEROL/HREF=\"$realm{LnkRoll}GENPARAM\" TARGET=\"DIEROL\" TABINDEX=\"6\" GENJSCODE/gs;
 			$panel =~ s/LWHOSON/HREF=\"$realm{LnkWho}GENPARAM\" TARGET=\"WHOSON\" TABINDEX=\"7\" GENJSCODE/gs;

			# Helper links
 			$panel =~ s/LCOLORS/HREF=\"$realm{LnkColors}GENPARAM\" TARGET=\"REALMS\" ACCESSKEY=\"g\" GENJSCODE/gs;
			$panel =~ s/LZOOMER/HREF=\"$realm{LnkZoom}GENPARAM\" TARGET=\"ZOOMER\" ACCESSKEY=\"z\" GENJSCODE/gs;


			# Updating
			print " Updating..";

			# Changes to handler
			$database->Quick("UPDATE $site.RealmDesign
				SET DesignContent=?
				WHERE Realmname=?
				  AND TagName=?", $panel, $realm, 'RealmPanel');

			print " Done!\n";
		}

		# Spacer
		print "\n\n";
	}

	print "Ethereal Realms Comm Panel conversion interface:\n";
	print "Usage: update_comm.pl {all|schema}\n\n";
